<?php
// radiobox 0.3 - PHP SHOUTcast scraper. mailto:and@doxdesk.com. Use freely

// Error messages
//
define('RADIOBOX_OFFLINEMSG', '(Radio currently off-line)');
define('RADIOBOX_DEADMSG', '(Server not responding)');

// radiobox_query - Scrape information from a SHOUTcast server status page,
// return as an array of properties.
//
function radiobox_query($server, $port) {
    $TD0= ': </font></td><td><font class=default>';
    $TD1= '</td>';

    // initialise return value
    //
    $r= array();
    $r['online']= false;
    $r['error']=$r['title']=$r['type']=$r['genre']= '';
    $r['url']=$r['aim']=$r['icq']=$r['irc']=$r['song']=$r['time']= '';
    $r['bitrate']=$r['listeners_max']= 0;
    $r['listeners_total']=$r['listeners_unique']=$r['listeners_peak']= 0;

    // connect to server
    $response= radiobox_get($server, $port, '/');
    if ($response[0]!=200) {
        $r['error']= $response[1];
        return $r;
    }

    // check response page
    $html= $response[1];
    $status= radiobox_extract($html, 'Server Status'.$TD0, $TD1);
    $status= radiobox_extract($status, 'Server is currently ', ' and');
    if ($status!='up') {
        $r['error']= RADIOBOX_OFFLINEMSG;
        return $r;
    }
    $r['online']= true;

    // scrape metadata from it
    $r['title']= radiobox_extract($html, 'Stream Title'.$TD0, $TD1);
    $r['type']= radiobox_extract($html, 'Content Type'.$TD0, $TD1);
    $r['genre']= radiobox_extract($html, 'Stream Genre'.$TD0, $TD1);
    $url= radiobox_extract($html, 'Stream URL'.$TD0, $TD1);
    if (radiobox_checkurl($url))
        $r['url']= radiobox_extract($html, 'Stream URL'.$TD0, $TD1);
    $r['icq']= radiobox_extract($html, 'Stream ICQ'.$TD0, $TD1);
    $r['aim']= radiobox_extract($html, 'Stream AIM'.$TD0, $TD1);
    $r['irc']= radiobox_extract($html, 'Stream IRC'.$TD0, $TD1);
    $r['song']= radiobox_extract($html, 'Current Song'.$TD0, $TD1);
    $r['time']= radiobox_extract($html, 'Average Listen Time'.$TD0, $TD1);

    // scrape listener numbers
    $listeners= radiobox_extract($html, 'Stream Status'.$TD0, $TD1);
    $r['bitrate']= 0+radiobox_extract($listeners, 'Stream is up at ', ' kbps');
    $r['listeners_total']= 0+radiobox_extract($listeners, 'with ', ' of');
    $r['listeners_max']= 0+radiobox_extract($listeners, 'of ', ' listeners');
    $r['listeners_unique']= 0+radiobox_extract($listeners, '(', ' unique)');
    $r['listeners_peak']= 0+radiobox_extract($html, 'Listener Peak'.$TD0, $TD1);

    return $r;
}


// radiobox_checkurl - Determine whether a URL is OK to include on the page.
// Don't show dangerous URLs or the pointless ones put in as default values
// by some source clients.
//
function radiobox_checkurl($url) {
    // get hostname - not using parse_url as it's flaky
    list($scheme, $url)= radiobox_plode('://', $url, 2);
    list($host, $path)= radiobox_plode('/', $url, 2);

    if ($scheme!='http' && $scheme!='https') return false;
    if (strpos($host, '@')!==false || strpos($host, ':')!==false) return false;
    if ($path=='' && (
        $host=='www.shoutcast.com'  || $host=='www.oddsock.org' ||
        $host=='www.audiorealm.com' || $host=='www.icecast.org' ||
        $host=='www.virtualdj.com'  || $host=='www.rogueamoeba.com'
    )) return false;
    return true;
}


// General-purpose functions __________________________________________________

// radiobox_get - Timeout-safe HTTP GET
// Returns an array of [integer status, data or error string]
// Doesn't support anything complicated like chunked transfer-encoding, but
// then SHOUTcast doesn't use it.
//
function radiobox_get($server, $port, $path) {
    $TIMEOUT= 8;
    $UA= 'Mozilla/4.0 (a bit compatible; SHOUTcast radio scraper)';
    $CHUNK= 1024;
    $TICK= 1;

    // send request
    $socket= fsockopen($server, $port, $errno, $errstr, $TIMEOUT);
    if (!$socket) return array(500, "$DEADMSG ($errno)");
    stream_set_blocking($socket, FALSE);
    fputs($socket, "GET $path HTTP/1.0\r\nUser-Agent: $UA\r\n\r\n");

    // read response
    $data= '';
    $begin= gmtime();
    while (!feof($socket)) {
        $chunk= fgets($socket, $CHUNK);
        if ($chunk!='') {
            $data.= $chunk;
        } else {
            if (gmtime()-$begin>=$TIMEOUT)
                return array(500, RADIOBOX_DEADMSG);
            sleep($TICK);
        }
    }

    // extract status header
    list($header, $body)= radiobox_plode("\r\n\r\n", $data, 2);
    list($status, )= radiobox_plode("\r\n", $header, 2);
    list( , $code, $reason)= radiobox_plode(" ", $status, 3);
    $code= 0+$code;

    if ($code==200) return array($code, $body);
    else return array($code, "$reason ($code)");
}


// radiobox_extract - extract detagged text from between a prefix and suffix
//
function radiobox_extract($text, $prefix, $suffix) {
    $ENTTABLE= get_html_translation_table(HTML_SPECIALCHARS, ENT_QUOTES);

    $ix0= strpos($text, $prefix);
    if ($ix0===false) return '';
    $ix0+= strlen($prefix);
    $ix1= strpos($text, $suffix, $ix0);
    if ($ix1===false) return '';

    $html= strip_tags(substr($text, $ix0, $ix1-$ix0));
    return strtr($html, array_flip($ENTTABLE));
}


// radiobox_plode - explode, padding result list up to the limit
//
function radiobox_plode($separator, $haystack, $limit) {
    $plodes= explode($separator, $haystack, $limit);
    while (count($plodes)<$limit)
        array_push($plodes, '');
    return $plodes;
}
?>